<?php

/**
 * @file
 * Administrative form for QuickBooks settings.
 */

require_once ji_quickbooks_get_path('qbosdk3260') . '/config.php';
require_once ji_quickbooks_get_path('qbosdk3260') . '/Core/ServiceContext.php';
require_once ji_quickbooks_get_path('qbosdk3260') . '/DataService/DataService.php';
require_once ji_quickbooks_get_path('qbosdk3260') . '/PlatformService/PlatformService.php';
require_once ji_quickbooks_get_path('qbosdk3260') . '/Utility/Configuration/ConfigurationManager.php';
require_once ji_quickbooks_get_path('qbosdk3260') . '/Core/OperationControlList.php';

/**
 * JIQuickbooksService - provides access to QBO.
 */
class JIQuickBooksService {

  /**
   * Service stype.
   *
   * @var string
   */
  public $serviceType;

  /**
   * Realm Id.
   *
   * @var string
   */
  public $realmId;

  /**
   * Request Validator.
   *
   * @var OAuthRequestValidator
   */
  public $requestValidator;

  /**
   * Service Context.
   *
   * @var ServiceContext
   */
  public $serviceContext;

  /**
   * Data Service.
   *
   * @var DataService
   */
  public $dataService;
  public static $faultValidation = 0;
  public static $faultSevere = 1;
  public static $faultUncaught = 2;

  /**
   * Saves error message if QBO settings cannot be parsed.
   *
   * @var String
   */
  public $settingErrorMessage;

  /**
   * Setup QuickBooks using the App.config file.
   *
   * @param bool $email_error
   *   TRUE sends email if error occurs.
   */
  public function __construct($email_error = TRUE) {
    $this->getConnection($email_error);
  }

  /**
   * Creates one connection to QuickBooks Online.
   */
  private function getConnection($email_error) {

    // Specify QBO or QBD.
    $this->serviceType = IntuitServicesType::QBO;

    // Get App Config.
    $this->realmId = ConfigurationManager::AppSettings('RealmID');

    if (!$this->realmId) {
      $this->settingErrorMessage = t("Please add the RealmID to App.config or else JI QuickBooks cannot function.");
      watchdog('JI QuickBooks', $this->settingErrorMessage, NULL, WATCHDOG_CRITICAL);
      if ($email_error) {
        $this->sendErrorNoticeEmail($this->settingErrorMessage);
      }
      return;
    }

    // Prep Service Context.
    $this->requestValidator = new OAuthRequestValidator(
        ConfigurationManager::AppSettings('AccessToken'), ConfigurationManager::AppSettings('AccessTokenSecret'), ConfigurationManager::AppSettings('ConsumerKey'), ConfigurationManager::AppSettings('ConsumerSecret')
    );

    $this->serviceContext = new ServiceContext($this->realmId, $this->serviceType, $this->requestValidator);

    if (!$this->serviceContext) {
      $this->settingErrorMessage = t("Problem while initializing ServiceContext.");
      watchdog('JI QuickBooks', $this->settingErrorMessage, NULL, WATCHDOG_CRITICAL);
      if ($email_error) {
        $this->sendErrorNoticeEmail($this->settingErrorMessage);
      }
      return;
    }

    $this->serviceContext->minorVersion = 3;

    // Prep Data Services.
    $this->dataService = new DataService($this->serviceContext);

    if (!$this->dataService) {
      $this->settingErrorMessage = t("Problem while initializing DataService.");
      watchdog('JI QuickBooks', $this->settingErrorMessage, NULL, WATCHDOG_CRITICAL);
      if ($email_error) {
        $this->sendErrorNoticeEmail($this->settingErrorMessage);
      }
      return;
    }
  }

  /**
   * Adds or updates customer information.
   *
   * $customer_data array that maps to IPPCustomer.
   * Mapping is useful to help prevent creating multiple
   * functions when form submission or object submission is used.
   */
  public function processCustomer($customer_data = array()) {
    $o_customer = new IPPCustomer();

    // Not new, retrieve QBO Id and token so Add()
    // updates rather than inserts.
    if (!empty($customer_data['Id'])) {
      // Quickbooks requires we retrieve a SyncToken
      // first, if we're going to update.
      $customer_response = $this->getCustomerById($customer_data['Id']);

      $o_customer->Id = $customer_data['Id'];
      $o_customer->SyncToken = $customer_response->SyncToken;
    }

    $o_customer->sparse = TRUE;
    $o_customer->Title = $customer_data['Title'];
    $o_customer->GivenName = $customer_data['GivenName'];
    $o_customer->MiddleName = $customer_data['MiddleName'];
    $o_customer->FamilyName = $customer_data['FamilyName'];
    // Was missing. Needed?
    // $oCustomer->Suffix = $customer_data['Suffix']; save.
    $o_customer->FullyQualifiedName = $o_customer->Title . ' ' . $o_customer->GivenName . ' ' . $o_customer->MiddleName . ' ' . $o_customer->FamilyName;

    $o_customer->CompanyName = $customer_data['CompanyName'];
    $o_customer->DisplayName = $customer_data['DisplayName'];
    $o_customer->PrintOnCheckName = $o_customer->FullyQualifiedName;

    $o_customer->Notes = $customer_data['Notes'];

    $o_primary_phone = new IPPTelephoneNumber();
    $o_primary_phone->FreeFormNumber = $customer_data['PrimaryPhone'];
    $o_customer->PrimaryPhone = $o_primary_phone;

    if (!empty($customer_data['Fax'])) {
      $o_fax = new IPPTelephoneNumber();
      $o_fax->FreeFormNumber = $customer_data['Fax'];
      $o_customer->Fax = $o_fax;
    }
    else {
      $o_customer->Fax = $customer_response->Fax;
    }

    if (!empty($customer_data['Mobile'])) {
      $o_mobile = new IPPTelephoneNumber();
      $o_mobile->FreeFormNumber = $customer_data['Mobile'];
      $o_customer->Mobile = $o_mobile;
    }
    else {
      $o_customer->Mobile = $customer_response->Mobile;
    }

    if (!empty($customer_data['Email'])) {
      $o_primary_email_addr = new IPPEmailAddress();
      $o_primary_email_addr->Address = $customer_data['Email'];
      $o_customer->PrimaryEmailAddr = $o_primary_email_addr;
    }
    else {
      $o_customer->PrimaryEmailAddr = $customer_response->PrimaryEmailAddr;
    }

    if (!empty($customer_data['Website'])) {
      $o_web_addr = new IPPWebSiteAddress();
      $o_web_addr->URI = $customer_data['Website'];
      $o_customer->WebAddr = $o_web_addr;
    }
    else {
      $o_customer->WebAddr = $customer_response->WebAddr;
    }

    $o_bill_address = new IPPPhysicalAddress();
    $o_bill_address->Line1 = $customer_data['BillAddressStreet1'];
    $o_bill_address->Line2 = $customer_data['BillAddressStreet2'];
    $o_bill_address->City = $customer_data['BillAddressCity'];
    $o_bill_address->CountrySubDivisionCode = $customer_data['BillAddressCountrySubDivisionCode'];
    $o_bill_address->PostalCode = $customer_data['BillAddressPostalCode'];
    $o_bill_address->Country = $customer_data['BillAddressCountry'];
    // $oBillAddress->Lat = $request->BillAddr->Lat; save.
    // $oBillAddress->Long = $request->BillAddr->Long; save.
    $o_customer->BillAddr = $o_bill_address;

    $o_ship_address = new IPPPhysicalAddress();
    $o_ship_address->Line1 = $customer_data['ShipAddressStreet1'];
    $o_ship_address->Line2 = $customer_data['ShipAddressStreet2'];
    $o_ship_address->City = $customer_data['ShipAddressCity'];
    $o_ship_address->CountrySubDivisionCode = $customer_data['ShipAddressCountrySubDivisionCode'];
    $o_ship_address->PostalCode = $customer_data['ShipAddressPostalCode'];
    $o_ship_address->Country = $customer_data['ShipAddressCountry'];
    // $oShipAddress->Lat = $request->ShipAddr->Lat; save.
    // $oShipAddress->Long = $request->ShipAddr->Long; save.
    $o_customer->ShipAddr = $o_ship_address;

    $o_customer->Active = $customer_data['Active'];

    $response = array();
    // This may insert or update.
    $response['response'] = $this->dataService->Add($o_customer);
    $response['error'] = $this->checkErrors();
    return $response;
  }

  /**
   * Adds/Updates an invoice.
   *
   * $invoice_data array that maps to IPPInvoice.
   * Mapping is useful to help prevent creating multiple
   * functions when form submission or object submission is used.
   */
  public function processInvoice($invoice_data = array()) {
    $o_invoice = new IPPInvoice();

    // CustomerId.
    $o_invoice->CustomerRef = $invoice_data['CustomerRef'];
    $o_customer_memo_ref = new IPPMemoRef();
    $o_invoice->CustomerMemo = $o_customer_memo_ref;
    $o_invoice->TxnDate = $invoice_data['TxnDate'];
    $o_invoice->SalesTermRef = $invoice_data['SalesTermRef'];
    $o_invoice->BillEmail = $invoice_data['BillEmail'];
    $o_invoice->Line = $invoice_data['Line'];
    $o_invoice->TxnTaxDetail = $invoice_data['TxnTaxDetail'];
    $o_invoice->AllowOnlineCreditCardPayment = 0;
    $o_invoice->AllowOnlineACHPayment = 1;
    $o_invoice->AllowIPNPayment = 1;
    $o_invoice->AllowOnlinePayment = 1;

    $invoice = $this->dataService->Add($o_invoice);
    $this->checkErrors();

    // Flags QuickBooks to send email contaning an invoice.
    if ($invoice_data['EmailInvoice']) {
      $this->dataService->SendEmail($invoice);
      $this->checkErrors();
    }

    return $invoice;
  }

  /**
   * Queries QuickBooks for TaxCode name.
   */
  public function checkTaxName($name) {
    $name_checked = check_plain($name);
    return $this->dataService->Query("SELECT * FROM TaxCode where Name in ('$name_checked')");
  }

  /**
   * Queries QuickBooks for TaxRate name.
   */
  public function checkTaxRateName($name) {
    $name_checked = check_plain($name);
    return $this->dataService->Query("SELECT * FROM TaxRate where Name in ('$name_checked')");
  }

  /**
   * Queries QuickBooks for TaxAgency name.
   *
   * Either way we should receive a TaxAgency name.
   */
  public function checkAgencyAddAgencyName($name) {
    $name_checked = check_plain($name);
    $query_response = $this->dataService->Query("SELECT * FROM TaxAgency where Name = '$name_checked'");

    // New name, let's add it.
    if (!$query_response) {
      $o_tax_agency = new IPPTaxAgency();
      $o_tax_agency->DisplayName = $name;
      return $add_response = $this->dataService->Add($o_tax_agency);
    }

    return $query_response;
  }

  /**
   * Returns any errors returned by the QBO API.
   *
   * Used during form submission/validation. Will stop a
   * form from successfully adding or updating data in
   * Drupal if Quickbooks returns an error, if needed. Displays
   * error message as well.<br>
   */
  public function checkErrors() {
    $error = $this->dataService->getLastError();
    if (!is_null($error)) {
      $response_xml_obj = new SimpleXMLElement($error->getResponseBody());
      $type = (string) $response_xml_obj->Fault[0]->attributes()->type;
      $code = (string) $response_xml_obj->Fault[0]->Error[0]->attributes()->code;
      $element = (string) $response_xml_obj->Fault[0]->Error[0]->attributes()->element;
      $message = (string) $response_xml_obj->Fault[0]->Error[0]->Message[0];
      $detail = (string) $response_xml_obj->Fault[0]->Error[0]->Detail[0];

      // Severe errors do not stop form execution, validation ones do.
      // If the error is "severe" then a configuration error must have
      // occurred and an admin must address it (email is sent).
      switch ($code) {
        case 100:
          $message = t("QuickBooks said: Error 100. Please verify your RealmID within App.config - is it pointing to the correct company?");
          watchdog('ji_quickbooks', $message, NULL, WATCHDOG_CRITICAL);

          if (user_access('access quickbooks')) {
            drupal_set_message($message, 'error', FALSE);
          }

          $this->sendErrorNoticeEmail($message);

          return array(
            'message' => $message,
            'code' => $this::$faultSevere,
          );

        case 3100:
        case 3200:
          $message = t("QuickBooks said: Error @code ApplicationAuthenticationFailed. Please verify your App.config tokens and keys, they may have expired or were entered incorrectly.", array('@code' => $code));
          watchdog('ji_quickbooks', $message, NULL, WATCHDOG_CRITICAL);

          if (user_access('access quickbooks')) {
            drupal_set_message($message, 'error', FALSE);
          }

          $this->sendErrorNoticeEmail($message);

          return array(
            'message' => $message,
            'code' => $this::$faultSevere,
          );

        default:
          // Generally, a ValidationFault is an error in customer input.
          if ($type === 'ValidationFault') {
            // Ubercart forms work off form_set_error.
            if (module_exists('ubercart')) {
              form_set_error('', $element . ': ' . $detail);
            }

            // Commerce forms use their own special validation.
            if (module_exists('commerce')) {
              drupal_set_message($message, 'error', FALSE);
            }

            return array(
              'message' => $message,
              'code' => $this::$faultValidation,
            );
          }
      }

      watchdog('ji_quickbooks', $detail, NULL, WATCHDOG_CRITICAL);

      return array(
        'message' => t('Something went wrong.'),
        'code' => $this::$faultUncaught,
      );
    }

    return array(
      'message' => NULL,
      'code' => NULL,
    );
  }

  /**
   * Send admin an email.
   */
  public function sendErrorNoticeEmail($message) {
    if ($message === '') {
      return;
    }

    $params = array(
      'subject' => t('[JI QuickBooks] - Severe Error Encountered'),
      'body' => check_markup(
          $message, 'plain_text'
      ),
    );

    $site_email = variable_get('site_mail', '');

    drupal_mail('ji_quickbooks', 'error', $site_email, language_default(), $params);
  }

  /**
   * Query QBO to GetAllCustomers.
   */
  public function getAllCustomers() {
    return $this->dataService->FindAll('Customer');
  }

  /**
   * Query QBO to GetCustomerById.
   */
  public function getCustomerById($id) {
    return $this->dataService->FindById(new IPPCustomer(array('Id' => $id), TRUE));
  }

  /**
   * Query QBO to GetInvoiceById.
   */
  public function getInvoiceById($id) {
    return $this->dataService->FindById(new IPPInvoice(array('Id' => $id), TRUE));
  }

  /**
   * Query QBO to GetPaymentById.
   */
  public function getPaymentById($id) {
    return $this->dataService->FindById(new IPPPayment(array('Id' => $id), TRUE));
  }

  /**
   * Query QBO to GetCustomerById.
   */
  public function getTaxAgencies() {
    return $this->dataService->Query("SELECT * FROM TaxAgency");
  }

  /**
   * Query QBO to getTaxCodeById.
   */
  public function getTaxCodeById($id = NULL) {
    return $this->dataService->Query("SELECT * FROM TaxCode where Active in (true)");
  }

  /**
   * Query QBO to getTaxRateById.
   */
  public function getTaxRateById($id = NULL) {
    return $this->dataService->FindById(new IPPTaxRate(array('Id' => $id), TRUE));
  }

  /**
   * Returns all available taxes from QuickBooks.
   */
  public function getAllTaxes() {

    $tax_code = $this->dataService->Query("SELECT * FROM TaxCode where Active in (true,false)");
    $tax_rate = $this->dataService->Query("SELECT * FROM TaxRate where Active in (true,false)");
    $tax_agency = $this->dataService->Query("SELECT * FROM TaxAgency");

    $result = array();

    if (!$tax_code) {
      return;
    }

    foreach ($tax_code as $value_tax_code) {
      $o_tax_response = new IPPTaxCode();
      $o_tax_response->Id = $value_tax_code->Id;
      $o_tax_response->Name = $value_tax_code->Name;
      $o_tax_response->Active = $value_tax_code->Active;
      // Used to compare if two tax records are similar.
      $o_tax_response->MetaData = $value_tax_code->MetaData;

      $tax_rates = array();
      $count_tax_rate = 0;

      if (!is_array($value_tax_code->SalesTaxRateList->TaxRateDetail)) {
        $value_tax_code->SalesTaxRateList->TaxRateDetail = array($value_tax_code->SalesTaxRateList->TaxRateDetail);
      }

      foreach ($tax_rate as $value_tax_rate) {
        foreach ($value_tax_code->SalesTaxRateList->TaxRateDetail as $value_tax_rate_detail) {
          if ($value_tax_rate_detail->TaxRateRef == $value_tax_rate->Id) {
            $o_rate_response = new stdClass();
            $o_rate_response->TaxRateRef = $value_tax_rate->Id;
            $o_rate_response->Name = $value_tax_rate->Name;
            $o_rate_response->RateValue = $value_tax_rate->RateValue;
            $o_rate_response->AgencyRef = $value_tax_rate->AgencyRef;

            foreach ($tax_agency as $agency) {
              if ($agency->Id == $value_tax_rate->AgencyRef) {
                $o_rate_response->AgencyName = $agency->DisplayName;
              }
            }

            $tax_rates[$count_tax_rate] = $o_rate_response;
            $count_tax_rate++;
          }
        }
      }

      $o_tax_response->TaxRates = $tax_rates;

      $result[$o_tax_response->Id] = $o_tax_response;
    }

    return $result;
  }

  /**
   * Query QBO to getAllTaxCodes.
   */
  public function getAllTaxCodes() {
    return $this->dataService->Query("SELECT * FROM TaxCode where Active in (true,false)");
  }

  /**
   * Query QBO to getAllTaxRates.
   */
  public function getAllTaxRates() {
    return $this->dataService->Query("SELECT * FROM TaxRate where Active in (true,false)");
  }

  /**
   * Query QBO to getAllTaxAgency.
   */
  public function getAllTaxAgency() {
    return $this->dataService->Query("SELECT * FROM TaxAgency");
  }

  /**
   * Query QBO to GetAllProducts or Items.
   */
  public function getAllProducts() {
    return $this->dataService->Query('SELECT * FROM Item');
  }

  /**
   * Query QBO to GetAllTerms.
   */
  public function getAllTerms() {
    return $this->dataService->Query('SELECT * FROM Term WHERE Active=TRUE ORDERBY Id');
  }

  /**
   * Query QBO to GetAllAccounts.
   */
  public function getAllAcounts() {
    return $this->dataService->Query("SELECT * FROM Account");
  }

  /**
   * Query QBO to getAcountsByType.
   */
  public function getAcountsByType($types = array()) {
    $filter = '';
    foreach ($types as $key => $type) {
      $types[$key] = "'" . $type . "'";
    }

    if (!empty($types)) {
      $filter = " WHERE AccountType in(" . implode(', ', $types) . ")";
    }

    return $this->dataService->Query("SELECT * FROM Account" . $filter);
  }

  /**
   * Query QBO to GetAllPaymentMethods.
   */
  public function getAllPaymentMethods() {
    return $this->dataService->Query("SELECT * FROM PaymentMethod");
  }

}
