<?php

/**
 * @file
 * Administrative form for QuickBooks settings.
 */

/**
 * Builds the main settings page.
 */
function ji_quickbooks_admin_main_settings() {

  // If the module is not yet installed guide them to the right path.
  $ji_quickbooks = ji_quickbooks_get_path('qbosdk3260');
  if (empty($ji_quickbooks)) {
    drupal_set_message('The QuickBooks SDK is missing.  Please install that first.  Click ' . l(t('here'), '/admin/help/ji_quickbooks') . ' for more information.', 'error', FALSE);

    $form['ji_quickbooks'] = array(
      '#type' => 'fieldset',
      '#title' => t('QuickBooks Settings'),
      '#description' => t("Please follow these steps in order to complete the installation:"),
      '#collapsible' => FALSE,
      '#tree' => TRUE,
    );
    $list_items = array(
      'Download SDK from ' . l(t('Josh Ideas'), 'https://joshideas.com/sdk/ji_quickbooks/qbosdk3260.zip')
      . ' and place in the libraries directory (/sites/all/libraries/).',
      'The installation of OAuth on your web server using the php.ini extension=oauth.so is required. '
      . "The module will alert you if it detects OAuth isn't installed on the web server.<br>",
    );

    $form['ji_quickbooks']['payment_method'] = array(
      '#prefix' => '<ul><li>',
      '#markup' => implode('</li><li>', $list_items),
      '#suffix' => '</li></ul>',
    );
    return $form;
  }

  // Won't load the screen if OAuth isn't detected.
  module_load_include('inc', 'ji_quickbooks', 'ji_quickbooks_support');
  if (!ji_quickbooks_support_check_oauth(TRUE)) {
    return;
  }

  $quickbooks_service = new JIQuickBooksService(FALSE);
  if (!empty($quickbooks_service->settingErrorMessage)) {
    drupal_set_message($quickbooks_service->settingErrorMessage, 'error', FALSE);
    return;
  }

  // We retrieve only the accounts of type 'Other Current Asset' and 'Bank'
  // as per documentation QBO does allow ony those 2 types for payment
  // see https://developer.intuit.com/docs/api/accounting/payment
  $account_array['accounts'] = $quickbooks_service->getAcountsByType(array('Other Current Asset', 'Bank'));
  $account_array['payment_methods'] = $quickbooks_service->getAllPaymentMethods();
  $account_array['terms'] = $quickbooks_service->getAllTerms();
  $account_array['products'] = $quickbooks_service->getAllProducts();

  $error_data = $quickbooks_service->checkErrors();
  $accounts = array();
  $accounts[0] = 'Please select...';

  $pay_methods = array();
  $pay_methods[0] = 'Please select...';

  $term = array();
  $term[0] = 'Please select...';

  $products = array();
  $products[0] = 'Please select...';

  if (empty($error_data['code'])) {
    foreach ($account_array['accounts'] as $account) {
      $accounts[$account->Id] = $account->Name;
    }
    foreach ($account_array['payment_methods'] as $pay_method) {
      $pay_methods[$pay_method->Id] = $pay_method->Name;
    }
    foreach ($account_array['terms'] as $terms) {
      $term[$terms->Id] = $terms->Name;
    }
    foreach ($account_array['products'] as $product) {
      $products[$product->Id] = $product->Name;
    }
  }

  $form['ji_quickbooks'] = array(
    '#type' => 'fieldset',
    '#title' => t('QuickBooks Settings'),
    '#description' => t("These are pulled from QuickBooks and during checkout, are applied to invoices and payments."),
    '#collapsible' => FALSE,
    '#tree' => TRUE,
  );
  $form['ji_quickbooks']['payment_method'] = array(
    '#type' => 'select',
    '#title' => t('Payment Method'),
    '#description' => t("When a payment is created; which method should we tell QBO is used? Hint: Probably Credit Card."),
    '#options' => $pay_methods,
    '#default_value' => variable_get('ji_quickbooks_payment_method', 0),
    '#required' => TRUE,
  );
  $form['ji_quickbooks']['account_payment'] = array(
    '#type' => 'select',
    '#title' => t('Account Selected'),
    '#description' => t("When payment is created, to what business account should QuickBooks apply it to?"),
    '#options' => $accounts,
    '#default_value' => variable_get('ji_quickbooks_payment_account', 0),
    '#required' => TRUE,
  );
  $form['ji_quickbooks']['terms'] = array(
    '#type' => 'select',
    '#title' => t('Terms'),
    '#description' => t("Choose the due date for an invoice. Currently, payments are applied immediately after invoice is created."),
    '#options' => $term,
    '#default_value' => variable_get('ji_quickbooks_term', 0),
    '#required' => TRUE,
  );
  $form['ji_quickbooks']['product'] = array(
    '#type' => 'select',
    '#title' => t('Products'),
    '#description' => t("Which QuickBooks Products or Services should all invoice line items relate to?"),
    '#options' => $products,
    '#default_value' => variable_get('ji_quickbooks_default_product', 0),
    '#required' => TRUE,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save settings'),
  );
  return $form;
}

/**
 * Saves settings/parameters to variables.
 */
function ji_quickbooks_admin_main_settings_submit($form = array(), &$form_state = array()) {
  variable_set('ji_quickbooks_payment_method', $form_state['values']['ji_quickbooks']['payment_method']);
  variable_set('ji_quickbooks_payment_account', $form_state['values']['ji_quickbooks']['account_payment']);
  variable_set('ji_quickbooks_term', $form_state['values']['ji_quickbooks']['terms']);
  variable_set('ji_quickbooks_default_product', $form_state['values']['ji_quickbooks']['product']);
}
