<?php

/**
 * @file
 * Helper functions.
 */

/**
 * Name of the cached table.
 */
define('JI_QUICKBOOKS_TAX_AGENCIES', 'ji_quickbooks_tax_agencies');
/**
 * Caches QBO responses since ji_autocomplete makes multiple calls.
 *
 * Probably the coolest function on the planet! Prevents
 * multiple calls to QBO and instead caches results (db storage)
 * which improves performance.
 */
function ji_quickbooks_tax_agencies_cache() {
  $option_agencies = &drupal_static(__FUNCTION__);
  if (!isset($option_agencies)) {
    $cache = cache_get(JI_QUICKBOOKS_TAX_AGENCIES);
    if ($cache) {
      $option_agencies = $cache->data;
    }
    else {
      $quickbooks_service = new JIQuickBooksService();
      $response = $quickbooks_service->getTaxAgencies();
      $error = $quickbooks_service->checkErrors();
      if (!empty($error['code'])) {
        return TRUE;
      }

      $options = array();
      foreach ($response as $agency) {
        $options[$agency->Id] = $agency->DisplayName;
      }
      cache_set(JI_QUICKBOOKS_TAX_AGENCIES, $options, 'cache');

      return $options;
    }
  }
  return $option_agencies;
}

/**
 * Checks against so common errors and returns execution.
 *
 * @return bool
 *   FALSE if no errors true otherwise.
 */
function ji_quickbooks_check_common_errors() {
  // If OAuth isn't installed, don't allow our functions to run since it will
  // crash general functionality.
  $oauth = ji_quickbooks_support_check_oauth(TRUE);
  if (!$oauth) {
    return TRUE;
  }

  // If the user forgets to adjust the settings, inform them
  // if user is an admin and return.
  // Prevents the system from crashing execution.
  $settings_set = ji_quickbooks_support_settings_applied(TRUE);
  if (!$settings_set) {
    return TRUE;
  }

  return FALSE;
}

/**
 * Support function that returns a specific value from an array.
 */
function ji_quickbooks_support_get_from_options($id, $options = array()) {
  return $options[$id];
}

/**
 * Checks if all settings for JI QuickBooks are set.
 *
 * Returns boolean false if even one of the settings
 * haven't been applied.
 */
function ji_quickbooks_support_settings_applied($show_message = FALSE) {
  $settings_count = 0;
  variable_get('ji_quickbooks_payment_method') ? $settings_count++ : NULL;
  variable_get('ji_quickbooks_payment_account') ? $settings_count++ : NULL;
  variable_get('ji_quickbooks_term') ? $settings_count++ : NULL;
  variable_get('ji_quickbooks_default_product') ? $settings_count++ : NULL;
  if (empty($settings_count) || $settings_count < 4) {
    if (user_access('access quickbooks')) {
      if ($show_message) {
        drupal_set_message(t('JI QuickBooks is missing the one or more settings. Please make adjustments or save before you use it. Visit the <a href="@url">settings page</a>.', array('@url' => url('admin/config/services/ji_quickbooks'))), 'error', FALSE);
      }
    }

    $url = url('admin/config/services/ji_quickbooks');
    watchdog('JI QuickBooks', "JI QuickBooks is missing the one or more settings. Please make adjustments or save before you use it. Visit the <a href='$url'>settings page</a>.", NULL, WATCHDOG_CRITICAL);

    return FALSE;
  }

  return TRUE;
}

/**
 * Checks if OAuth is installed/enabled on server.
 *
 * $show_message boolean. Note: Will only show message if user has
 * permission "administer site configuration".
 */
function ji_quickbooks_support_check_oauth($show_message = FALSE) {
  $oauth = extension_loaded('oauth');
  if (!$oauth) {
    if (user_access('access quickbooks')) {
      if ($show_message) {
        $internal_link = t('JI QuickBooks needs OAuth installed on web server.  <a href="@link">See Status Report</a>.', array(
          '@link' => url('/admin/reports/status'),
        ));
        drupal_set_message($internal_link, 'error', FALSE);
      }
    }

    return FALSE;
  }

  return TRUE;
}

/**
 * Gets the path of a library.
 */
function ji_quickbooks_get_path($name, $base_path = FALSE) {
  $libraries = &drupal_static(__FUNCTION__);

  if (!isset($libraries)) {
    $libraries = ji_quickbooks_get_libraries();
  }

  $path = ($base_path ? base_path() : '');
  if (!isset($libraries[$name])) {
    return FALSE;
  }
  else {
    $path .= $libraries[$name];
  }

  return $path;
}

/**
 * Loads all libraries.
 */
function ji_quickbooks_get_libraries() {
  $searchdir = array();
  $profile = drupal_get_path('profile', drupal_get_profile());
  $config = conf_path();

  // Similar to 'modules' and 'themes' directories in the root directory,
  // certain distributions may want to place libraries into a 'libraries'
  // directory in Drupal's root directory.
  $searchdir[] = 'libraries';

  // Similar to 'modules' and 'themes' directories inside an installation
  // profile, installation profiles may want to place libraries into a
  // 'libraries' directory.
  $searchdir[] = "$profile/libraries";

  // Always search sites/all/libraries.
  $searchdir[] = 'sites/all/libraries';

  // Also search sites/<domain>/*.
  $searchdir[] = "$config/libraries";

  // Retrieve list of directories.
  $directories = array();
  $nomask = array('CVS');
  foreach ($searchdir as $dir) {
    if (is_dir($dir) && $handle = opendir($dir)) {
      while (FALSE !== ($file = readdir($handle))) {
        if (!in_array($file, $nomask) && $file[0] != '.') {
          if (is_dir("$dir/$file")) {
            $directories[$file] = "$dir/$file";
          }
        }
      }
      closedir($handle);
    }
  }

  return $directories;
}

/**
 * Ensures a standard formatted name is returned.
 *
 * 1) Retrieves $qbo_customer_id if available or returns an empty string<br>
 * 2) Retrieves $display_name by creating from anonymous information or<br>
 * existing $user object information<br>
 * Note: When adding a new customer to QuickBooks,<br>
 * $display_name must be unique since QuickBooks doesn't<br>
 * allow duplicates. It's okay to have the same $display_name<br>
 * if we're updating a customer.
 */
function ji_quickbooks_user_set_name_id($account = NULL, $email = NULL) {
  $qbo_customer_id = '';
  $display_name = '';

  // Activate/Deactivates new customer accounts in
  // Quickbooks to avoid filling up the customer screen
  // with potential users who abandon their carts. NOTE: must
  // be a string NOT a boolean (requirement in Quickbooks API).
  $activate_user = 'TRUE';

  // User just completed the checkout process, use $account
  // object instead of $user object.
  if (!is_null($account)) {
    $qbo_customer_id = $account->data['qbo_customer_id'];
    $display_name = _ji_quickbooks_display_name_format($account->name, $account->uid);
  }
  else {
    // User is logged in.
    if (!user_is_anonymous()) {
      global $user;
      // A logged in user may have never purchased.
      if (isset($user->data['qbo_customer_id'])) {
        $qbo_customer_id = $user->data['qbo_customer_id'];
        $activate_user = 'TRUE';
      }
      else {
        $activate_user = 'FALSE';
      }
      $display_name = _ji_quickbooks_display_name_format($user->name, $user->uid);
    }
    else {
      $activate_user = 'FALSE';

      // Check if the email is used by an existing drupal user.
      if ($email) {
        $user = user_load_by_mail($email);
        if ($user) {
          $qbo_customer_id = $user->data['qbo_customer_id'];
          $display_name = _ji_quickbooks_display_name_format($user->name, $user->uid);
          $activate_user = 'TRUE';
        }
      }

      if ($activate_user === 'FALSE') {
        // The anonymous user returned with the same
        // session that already sent data to Quickbooks.
        // Now we update existing Quickbooks data.
        if (isset($_SESSION['qbo_customer_id'])) {
          $qbo_customer_id = $_SESSION['qbo_customer_id'];
        }

        // We use the cart contents order ID as temporary
        // unique number. This prevents duplication errors
        // when DisplayName hits Quickbooks.
        if (module_exists('uc_cart')) {
          $key = key(uc_cart_get_contents());
        }

        if (module_exists('commerce')) {
          $key = commerce_cart_order_id();
        }
        $display_name = _ji_quickbooks_display_name_format('anonymous user', $key);
      }
    }
  }

  return array(
    'qbo_customer_id' => $qbo_customer_id,
    'display_name' => $display_name,
    'is_new' => $activate_user,
  );
}

/**
 * Returns a standard format.
 */
function _ji_quickbooks_display_name_format($name, $id) {
  return "$name id$id";
}
